#!/usr/bin/env python
"""
Copyright (C) 2000-2005 Andrew Kenneth Milton. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import os
import sys
import cgi
import xmlrpclib

from cStringIO import StringIO

# Debugging
# import cgitb; cgitb.enable()

# Directory where you want files to be placed temporarily
# Make sure it exists!
prefixDir = '/home/webpages/zope/tmp'

# Maximum upload length (10 Mb)
maxUploadLength = 10000000

# Where to redirect when the upload is finished
# Will be passed all the form items, with files -> filenames
redirectURL = 'http://xanadu:8080/akm/IAmDone'

# Update URL -- path to the BatchUploader product
# We call setProgress and other methods on this directly.
updateURL = 'http://xanadu:8080/akm/uManager'

########################################################################

def setupXMLRPC(session):
	newURL = updateURL + '/%s'%(session)
	newServer = xmlrpclib.ServerProxy(newURL)
	return newServer

def exit(message):
	print "Content-type: text/html\n"
	print "<br>%s</br>\n"%(message)

def main():
	contentType = os.environ["CONTENT_TYPE"]
	contentLength = int(os.environ["CONTENT_LENGTH"])
	path = os.environ['PATH_TRANSLATED']
	sessionId = path[path.rfind('/')+1:]
	
	if contentLength > maxUploadLength:
		exit("Max upload limit exceeded.")
		return

	try:
		zopeServer = setupXMLRPC(sessionId)
	except:
		exit("Cannot contact Zope server.")
		return

	try:
		# Main loop to read in the post stuff...
		buffer = ''

		loops = 0
		while len(buffer) < contentLength:
			buffer += sys.stdin.read(8192) #8K chunking should be fine...
			if not (loops % 10):
				zopeServer.setUploadedAmount(len(buffer), contentLength)
			loops += 1

		# Set display to 100%
		zopeServer.setUploadedAmount(contentLength, contentLength)

		# Fake out the CGI lib to re-read our buffer as stdin so we can
		# rewrite the request and send it on to Zope.

		newBuffer = StringIO(buffer)
		sys.stdin = newBuffer
		form = cgi.FieldStorage()

		params = {}
		for k in form.keys():
			if form[k].file and form[k].filename:
				newFileName = os.tempnam(prefixDir, sessionId+'-'+k+'-')
				params[k] = newFileName
				f = open(newFileName, 'wb')
				f.write(form[k].file.read())
				f.close()
			else:
				params[k] = form.getvalue(k)

		qs = []
		for k, v in params.items():
			qs.append('%s=%s'%(k, cgi.escape(v)))

		qs = '&'.join(qs)

		finalURL = redirectURL + '?' + qs
		zopeServer.setDone()
	except:
		# Make cuter error message here...
		zopeServer.setError('An Error Occurred...')
	print "Location: %s\n\n"%(finalURL)

if __name__ == '__main__':
	main()
	sys.stdin.close()
