"""
Copyright (C) 2000-2005 Andrew Kenneth Milton. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""
import popen2
import os
from Globals import DTMLFile, Acquisition
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from threading import Thread
from time import sleep, time
from OFS.DTMLMethod import decapitate

import traceback
import sys

from uuidgen import uuidgen

class UploadThreadStatus:
	def __init__(self):
		self.completed = 0.0
		self.status = True # ok
		self.file = None
		self.isDone = False
		self.errorMessages = []
		self.statusMessage = None
		self.startTime = time()
		self.lastUpdated = self.startTime

	def setCompletedPercent(self, completed):
		self.lastUpdated = time()
		self.completed = completed

	def setReceiptLocation(self, s):
		self.receiptLocation = s

	def setErrorStatus(self):
		self.status = False

	def setGoodStatus(self):
		self.status = True

	def setDone(self):
		self.isDone = True

	def addErrorMessage(self, message):
		self.errorMessages.append(message)

	def setOtherStatusMessages(self, message):
		self.statusMessage = message

	def getOtherStatusMessages(self):
		return self.statusMessage

class DownloadThreadStatus(UploadThreadStatus):
	def setFileDownload(self, fileBuffer):
		self.file = fileBuffer

# Module Global, probably shouldn't be if there's going to be
# many instances.
ActiveDownloadThreads = {}
ActiveUploadThreads = {}

def getDownloadStatus(threadId):
	if ActiveDownloadThreads.has_key(threadId):
		return ActiveDownloadThreads[threadId]['threadStatus']
	return None

def getUploadStatus(threadId):
	if ActiveUploadThreads.has_key(threadId):
		return ActiveUploadThreads[threadId]['threadStatus']
	return None

def hasActiveDownloadThread(threadId):
	return ActiveDownloadThreads.has_key(threadId)

def hasActiveUploadThread(threadId):
	return ActiveUploadThreads.has_key(threadId)
			
def deleteUploadThread(threadId):
	del ActiveUploadThreads[threadId]
	
def deleteDownloadThread(threadId):
	del ActiveDownloadThreads[threadId]

class UploadStatus(Acquisition.Implicit):
	""" Upload Status Display """
	__allow_access_to_unprotected_subobjects__=1

	index_html__roles__=('Anonymous',)
	index_html=PageTemplateFile('zpt/UploadStatus', globals())
	
	def __init__(self, id):
		self.id = id
		self.status = getUploadStatus(id)

	def isDone(self):
		return self.status.isDone

	def percentComplete(self):
		""" I am web callable """
		return int(float(self.status.completed)*100)

	def getErrorMessages(self):
		return self.status.errorMessages

	def getStatus(self):
		return self.status.status

	def getOtherStatusMessages(self):
		return self.status.getOtherStatusMessages()

	def setPercentComplete(self, complete):
		""" I am callable from the web (XMLRPC) """
		self.status.setCompletedPercent(complete)

	def setUploadedAmount(self, soFar, total):
		""" I am callable from the web (XMLRPC) """
		perc = float(soFar) / float(total)
		self.setPercentComplete(perc)

	def setError(self, message):
		""" I am callable from the web (XMLRPC) """
		self.status.addErrorMessage(message)

	def setDone(self):
		""" I am callable from the web (XMLRPC) """
		self.status.setDone()

	# Make your code call this when it's done with the file upload so
	# the upload status disappear. Doesn't delete any uploaded files..
	def uploadComplete(self):
		deleteUploadThread(self.id)

	def getTimeDelta(self):
		now = time()
		return now - self.status.lastUpdated
	
class DownloadStatus(UploadStatus):
	""" Download Status Display """
	__allow_access_to_unprotected_subobjects__=1

	index_html__roles__=('Anonymous',)
	index_html=PageTemplateFile('zpt/DownloadStatus', globals())

	def __init__(self, id):
		self.id = id
		self.status = getDownloadStatus(id)

	def _getType(self):
		return "text/plain"
		
	def downloadFile(self, REQUEST):
		""" Must be web callable """
		have_key = REQUEST.RESPONSE.headers.has_key
		if not (have_key('content_type') or have_key('Content-Type')):
			c = self._getType()
			REQUEST.RESPONSE.setHeader('Content-Type', c)

		REQUEST.RESPONSE.setHeader('Content-Disposition','inline;filename=somefile')
		file = self.status.file
		deleteDownloadThread(self.id)
		return decapitate(file, REQUEST.RESPONSE)
	

from OFS.Folder import Folder
class TraversalError(Folder):
	meta_type = 'Traversal Error'
	objectType = 'Traversal Error'
	factoryType = 'Traversal Error'

	error_page = PageTemplateFile('zpt/DownloadExpired', globals())

	# Don't do anything yet, we can make it fancy later and provide a
	# redirection URL or something.
	def __init__(self):
		pass

	# Tell zope we want it to use __call__ instead of index_html
	index_html = None

	# Anyone can see us.
	__call____roles__ = None
	def __call__(self, client = None, REQUEST = None, RESPONSE = None):
		""" Traversal Error (404 most likely) """
		return self.error_page(self, client, REQUEST = REQUEST,
							   RESPONSE = RESPONSE)
        
	def _getOb(self, id, default=[]):
		# All Roads lead to Rome...
		#
		# In other words all further traversal past this point means we
		# return ourselves, so there's no escaping the error page.
		return self

_marker = []

manage_addUploadManagerForm = DTMLFile('dtml/manage_addUploadManagerForm',
									   globals())

def manage_addUploadManager(self, id, title = '', REQUEST = {}):
	""" Call Me """
	o = UploadManager(id, title)
	self._setObject(id, o)

	if REQUEST:
		REQUEST.RESPONSE.redirect(self.absolute_url()+'/manage_workspace')
	return ''

# An Upload Manager
class UploadManager(Folder):
	""" An upload 'Manager' """
	
	def __init__(self, id, title = ''):
		self.id = id
		self.title = title

	index_html__roles__ = ('Manager',)
	index_html = PageTemplateFile('zpt/UploadsInProgress', globals())

	def newUpload(self):
		uploadId = uuidgen()
		threadStatus = UploadThreadStatus() # Not really a thread..
		ActiveUploadThreads[uploadId] = {'thread':None,
										 'threadStatus':threadStatus}
		return uploadId

	def _cleanStaleEntries(self):
		now = time()
		for k, v in ActiveUploadThreads.items():
			if v['threadStatus'].lastUpdated == v['threadStatus'].startTime:
				if now - v['threadStatus'].lastUpdated > 300: # has timed out. 5 mins and no upload started.
					deleteUploadThread(k)
			if v['threadStatus'].isDone:
				if now - v['threadStatus'].lastUpdated > 1800: # 30 mins since last updated and complete..
					deleteUploadThread(k)
		
	def objectIds(self, types = []):
		self._cleanStaleEntries()
		return ActiveUploadThreads.keys()

	def objectValues(self, types = []):
		objs = []
		statii = self.objectIds()
		for status in statii:
			s = self.__getOb(str(status))
			objs.append(s)
		return objs

	def __getOb(self, threadid):
		if not ActiveUploadThreads.has_key(threadid):
			return TraversalError().__of__(self)
		thread = UploadStatus(threadid)
		thread = thread.__of__(self)
		return thread

	def _getOb(self, id, default = _marker):
		return self.__getOb(str(id))


# We don't currently export the DownloadManager into Zope Land...
class DownloadManager(Folder):
	""" A Download 'Manager' """

	meta_type = 'Download Manager'
	
	def __init__(self, id, title = '', basePath = '/'):
		self.id = id
		self.title = title
		self.basePath = basePath

	def newDownload(self):
		downloadId = uuidgen()
		threadStatus = DownloadThreadStatus()
		kw = {} # make some keywords here
		# Create a download target..
		# SomeMethod would do some processing, and then provide
		# A file for downloading... this process may take some time
		# So we provide a 'status' display.
		newThread = Thread(target = SomeMethod, KWargs = kw)
		ActiveDownloadThreads[downloadId] = {'thread':newThread,
											 'threadStatus':threadStatus}

		newThread.setDaemon(1)
		newThread.start()
		return downloadId

	index_html__roles__ = ('Anonymous',)
	def index_html(self, client = None, REQUEST = None, RESPONSE = None):
		" I am web callable"
		if RESPONSE:
			RESPONSE.redirect('/')
		else:
			REQUEST.RESPONSE.redirect('/')
		return ''

	def objectIds(self, types = []):
		return ActiveDownloadThreads.keys()
	
	def objectValues(self, types = []):
		objs = []
		statii = self.objectIds()
		for status in statii:
			s = self.__getOb(str(status))
			objs.append(s)
		return objs

	def __getOb(self, threadid):
		if not ActiveDownloadThreads.has_key(threadid):
			return TraversalError().__of__(self)
		thread = DownloadStatus(threadid)
		thread = thread.__of__(self)
		return thread

	def _getOb(self, id, default = _marker):
		return self.__getOb(str(id))
