"""
Copyright (C) 2000-2005 Andrew Kenneth Milton. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""
import sys
import sets
import time
import socket
import random

NativeUUIDGen = False

# If your platform has a native uuidgen add it here...
if sys.platform == 'win32':
	try:
		from pythoncom import CreateGuid
		def uuidgen():
			x = CreateGuid()
			return str(x)[1:-1]
		NativeUUIGen = True
	except:
		pass

# Fake it out..
if not NativeUUIDGen:

	UUID_NODE_LEN = 6;

	class UUIDInternalTime:
		def __init__(self, low, mid, high):
			self.low = low
			self.mid = mid
			self.high = high

			self.ll = self.low | (self.mid << 32) | (self.high << 48)

		def setLL(self, ll):
			self.ll = ll
			self.low = (self.ll & 0xFFFFFFFFL)
			self.mid = (self.ll >> 32L) & 0xFFFFL
			self.high = ((self.ll >> 48L) & 0xFFFL ) | ( 1L << 12)

	class UUID:
		def __init__(self, timeLow = 0, timeMid = 0, timeHigh = 0,
					 seq = 0, node = [0,] * UUID_NODE_LEN):
			self.time = UUIDInternalTime(timeLow, timeMid, timeHigh)
			self.seq = seq
			self.node = node

		def __str__(self):
			_node = ''.join(map(lambda n:'%02x'%(n), self.node))
			return "%08x-%04x-%04x-%04x-%s"%(
				self.time.low , self.time.mid, self.time.high,
				self.seq, _node)

	uuid_last = UUID(0, 0, 0, 0, [0,] * UUID_NODE_LEN)

	def uuidNode():
		nodes = [None,] * UUID_NODE_LEN

		# It's unlikely we can get a MAC address, so try to grab the
		# IPv4 address (4 bytes) and generate 2 more random bytes
		try:
			host, aliases, addresses = socket.gethostbyaddr(socket.gethostname())
			addr = addresses[0]
			dotted = addr.split('.')
			nodes = map(int, dotted)
			rands = [int(random.random() * 256.0),
					 int(random.random() * 256.0) ]
			nodes = nodes + rands
		# If the above fails, generate a random multicast address
		except:
			for i in range(len(nodes)):
				nodes[i] = int(random.random() * 256.0)

		return nodes

	def uuidTime():
		uTime = 0x01B21DD213814000L
		uTime += int(time.time() * 10000000.0)

		return (uTime & ((1L << 60) - 1L))


	def uuidgen():
		global uuid_last

		uuid = UUID()
		uuid.node = uuidNode()
		xTime = uuidTime()

		if ( uuid_last.time.ll == 0L or
			 uuid_last.node[:4] != uuid.node[:4] ):
			uuid.seq = int(random.random() * 65536.0) & 0x3FFF
		elif uuid_last.time.ll >= xTime:
			uuid.seq = (uuid_last.seq + 1) & 0x3FFF
		else:
			uuid.seq = uuid_last.seq


		uuid_last = uuid
		uuid_last.time.setLL((xTime) & (( 1L << 60) - 1L))

		uuid.seq = (uuid.seq | 0x8000)
		uuid.time.setLL(xTime)

		return str(uuid)


if __name__ == '__main__':
	print uuidgen()
